import { Recorder } from '../../../app';

/** Token的请求结果 */
export type ApiShortResult = {
  c: number;
  m: string;
  v: {
    appkey: string;
    token: string;
  };
};

/** 阿里云一句话识别配置 */
export class AliAsr {
  asr?: any;
  constructor() {}
  init() {
    this.asr = this.create();
  }
  create() {
    const asr = Recorder.ASR_Aliyun_Short({
      tokenApi: '/token',
      apiArgs: {
        //请求tokenApi时要传的参数
        action: 'token',
        lang: '普通话', //语言模型设置（具体取值取决于tokenApi支持了哪些语言）
      },
      compatibleWebSocket: null,
      //高级选项
      fileSpeed: 6, //单个文件识别发送速度控制，取值1-n；1：为按播放速率发送，最慢，识别精度完美；6：按六倍播放速度发送，花10秒识别60秒文件比较快，精度还行；再快测试发现似乎会缺失内容，可能是发送太快底层识别不过来导致返回的结果缺失。
      log: (msg, error) => {
        console.log('AliAsr log', msg, error);
      },
    });
    this.asr = asr;
    return asr;
  }
  /**
   * 获取输入的音频数据总时长
   * @returns
   */
  getInputDuration() {
    return this.asr?.inputDuration();
  }
  /**
   * 获取已发送识别的音频数据总时长
   * @returns
   */
  getSendDuration() {
    return this.asr?.sendDuration();
  }
  /**
   * 获取已识别的音频数据总时长
   * @returns
   */
  getAsrDuration() {
    return this.asr?.asrDuration();
  }
  /**
   * 获取实时结果文本
   * @returns
   */
  getText() {
    return this.asr?.getText();
  }
  /**
   * 一次性将单个完整音频Blob文件转成文字
   * @param audioBlob
   * @returns
   */
  audioToText(audioBlob: Blob) {
    return this.asr?.audioToText(audioBlob);
  }
  /**
   * 一次性将单个完整PCM音频数据转成文字
   * @param buffer
   * @param sampleRate
   * @returns
   */
  pcmToText(buffer: ArrayBuffer, sampleRate: number) {
    return this.asr?.pcmToText(buffer, sampleRate);
  }
  /**
   * 开始识别
   */
  start(recordStartFn: () => void, fail?: (msg?: string) => void) {
    this.asr?.start(recordStartFn, fail);
  }
  /**
   * 停止识别
   */
  stop(success?: (text: string, abortMsg?: string) => void, fail?: (msg?: string) => void) {
    this.asr?.stop(success, fail);
  }
}
