import { Recorder } from '../../../app';
import { AliAsr } from './AliAsr';

export class MyRecorder {
  private recorder?: typeof Recorder;
  private processTime = 0;
  private wave?: any;
  asr?: AliAsr;
  blob?: Blob;
  constructor() {}
  async init(asr?: AliAsr) {
    const that = this;
    this.asr = asr;
    this.recorder = new Recorder({
      type: 'mp3',
      sampleRate: 16000,
      bitRate: 16,
      onProcess: function (buffers, powerLevel, bufferDuration, bufferSampleRate, newBufferIdx, asyncEnd) {
        //录音实时回调，大约1秒调用12次本回调，buffers为开始到现在的所有录音pcm数据块(16位小端LE)
        //可利用extensions/sonic.js插件实时变速变调，此插件计算量巨大，onProcess需要返回true开启异步模式
        //可实时上传（发送）数据，配合Recorder.SampleData方法，将buffers中的新数据连续的转换成pcm上传，或使用mock方法将新数据连续的转码成其他格式上传，可以参考文档里面的：Demo片段列表 -> 实时转码并上传-通用版；基于本功能可以做到：实时转发数据、实时保存数据、实时语音识别（ASR）等
        that.processTime = Date.now();
        //可实时绘制波形（extensions目录内的waveview.js、wavesurfer.view.js、frequency.histogram.view.js插件功能）
        that.wave && that.wave.input(buffers[buffers.length - 1], powerLevel, bufferSampleRate);
        if (asr) {
          asr.asr?.input(buffers, bufferSampleRate, 0);
        }
      },
    });
    const isOpen = await this.open();
    console.log('open recorder', isOpen);
    return isOpen;
  }
  async open() {
    const that = this;
    return new Promise((resolve) => {
      that.recorder?.open(
        function () {
          if (Recorder.WaveView) that.wave = Recorder.WaveView({ elem: '.recwave' });
          console.log('open success');
          resolve(true);
        },
        function (msg, isUserNotAllow) {
          //用户拒绝未授权或不支持
          console.log('open fail', msg, isUserNotAllow);
          console.log((isUserNotAllow ? 'UserNotAllow，' : '') + '无法录音:' + msg);
          resolve(false);
        },
      );
    });
  }
  async startRecord() {
    let that = this;
    if (!this.recorder) {
      await this.init();
    }
    this.recorder?.start();
    this.processTime = 0;
    //【稳如老狗WDT】可选的，监控是否在正常录音有onProcess回调，如果长时间没有回调就代表录音不正常
    let rec = this.recorder;
    console.log(rec);
    rec.wdtPauseT = 0;
    const startTime = Date.now();
    const wdt = setInterval(function () {
      const processTime = that.processTime;
      console.log(rec, wdt, 'processTime', processTime);
      if (!rec || wdt != rec.watchDogTimer) {
        clearInterval(wdt);
        return;
      } //sync
      if (Date.now() < rec.wdtPauseT) return; //如果暂停录音了就不检测：puase时赋值rec.wdtPauseT=Date.now()*2（永不监控），resume时赋值rec.wdtPauseT=Date.now()+1000（1秒后再监控）
      if (Date.now() - (processTime || startTime) > 1500) {
        clearInterval(wdt);
        console.error(processTime ? '录音被中断' : '录音未能正常开始');
        // ... 错误处理，关闭录音，提醒用户
      }
    }, 1000);
    rec.watchDogTimer = wdt;
  }
  async stopRecord() {
    const that = this;
    let rec = this.recorder;
    rec.watchDogTimer = 0; //停止监控onProcess超时
    return new Promise((resolve) =>
      this.recorder?.stop(
        function (blob, duration) {
          const localUrl = (window.URL || webkitURL).createObjectURL(blob);
          console.log(blob, localUrl, '时长:' + duration + 'ms');
          rec.close(); //释放录音资源，当然可以不释放，后面可以连续调用start；但不释放时系统或浏览器会一直提示在录音，最佳操作是录完就close掉
          that.recorder = null;
          //已经拿到blob文件对象想干嘛就干嘛：立即播放、上传、下载保存
          that.blob = blob;
          /*** 【立即播放例子】 ***/
          const audio = document.createElement('audio');
          document.body.prepend(audio);
          audio.controls = true;
          audio.src = localUrl;
          audio.play();
          resolve(blob);
        },
        function (msg) {
          console.log('录音失败:' + msg);
          rec.close(); //可以通过stop方法的第3个参数来自动调用close
          rec = null;
          that.recorder = null;
          resolve(false);
        },
      ),
    );
  }
}
